/**************************************************************************************************
* NOTICE OF COPYRIGHT AND OWNERSHIP OF SOFTWARE:
*
* Copyright 2010 Flexera Software, Inc. and/or InstallShield Co. Inc.  All Rights Reserved.  
*
* This computer program is the property of Flexera Software, Inc. and InstallShield Co. Inc. of 
* Schaumburg, Illinois, U.S.A.  Any use, copy, publication, distribution, display, modification, 
* or transmission of this computer program in whole or in part in any form or by any means without
* the prior express written permission of Flexera Software, Inc. or InstallShield Co. Inc. is 
* strictly prohibited.
* 
* Except when expressly provided by Flexera Software, Inc. or InstallShield Co. Inc. in writing,
* possession of this computer program shall not be construed to confer any license or rights under 
* any of Flexera Software, Inc.s or InstallShield Co. Inc.s intellectual property rights, whether
* by estoppel, implication, or otherwise.  
* 
* ALL COPIES OF THIS PROGRAM MUST DISPLAY THIS NOTICE OF COPYRIGHT AND OWNERSHIP IN FULL.
**************************************************************************************************/
/*
	This header file is part of an FNP sample utility that demonstrates Composite Transactions.

	It declares the FlxCtuCommand (command) object and the functions that relate to it.
*/
#ifndef COMP_TRAN_UTIL_COMMAND_H
#define COMP_TRAN_UTIL_COMMAND_H

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "compTranUtilCommandDef.h"

typedef struct FlxCtuKeyValue_struct
{
	const char* pszKey;
	const char* pszValue;
} FlxCtuKeyValue;

typedef struct FlxCtuDictionary_struct
{
	const char* pszName;

	size_t	entryCount;
	size_t	entryIndex;
	FlxCtuKeyValue**		ppEntries;
} FlxCtuDictionary;

typedef struct FlxCtuAttribute_struct
{
	const FlxCtuAttributeDef*	pDef;
	FlxCtuKeyValue*				pKeyValue;
} FlxCtuAttribute;

typedef struct FlxCtuCommand_struct
{
	const FlxCtuCommandDef* pDef;
	const char*				pszNameForMessages;

	FlxActBool				bIsValue;
	const char*				pszValue;

	size_t					attributeCount;
	FlxCtuAttribute**		ppAttributes;

	size_t					dictionaryCount;
	FlxCtuDictionary**		ppDictionaries; 
} FlxCtuCommand;

FlxActBool flxCtuCommandAttributeIs(const FlxCtuCommand* pCommand, 
									const char*			 pszAttributeName,
									const char*			 pszAttributeValue);

FlxActBool flxCtuCommandAttributeIsYes(const FlxCtuCommand* pCommand, 
									   const char*			pszAttributeName,
									   FlxActBool			bDefault);

void flxCtuCommandDestroy(const FlxCtuCommand* pCommand);

const char* flxCtuCommandGetAttribute(const FlxCtuCommand* pCommand, const char* pszAttributeName);
const char* flxCtuCommandGetName(const FlxCtuCommand* pCommand);
const char* flxCtuCommandGetNameForMessages(const FlxCtuCommand* pCommand);
const char* flxCtuCommandGetValue(const FlxCtuCommand* pCommand);

FlxActBool flxCtuCommandHasAttribute(const FlxCtuCommand* pCommand, const char* pszAttributeName);
FlxActBool flxCtuCommandHasTrait(const FlxCtuCommand* pCommand, FlxCtuCommandTrait trait);
FlxActBool flxCtuCommandHasValue(const FlxCtuCommand* pCommand);

void flxCtuCommandPrintError(const FlxCtuCommand* pCommand, const char *pszFormatStr, ...);

void flxCtuDictionaryFree(const FlxCtuDictionary* pDictionary);
void flxCtuKeyValueFree(FlxCtuKeyValue* pKeyValue);


#endif	/* Include guard */
